<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01//EN"
  "http://www.w3.org/TR/html4/strict.dtd">
<html>
<head>
    <title>OpenID failed</title>
</head>
<body>
    <h1>OpenID failed</h1>
    <p>{{ message|escape }}</p>
</body>
</html>
